/**
 * 
 */
package thrust;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import rtflow.RTflowCode;

/**
 * Component that displays the game screen. It will automatically
 * read the game's state from a RTflowCode object when repainting,
 * but it will not automatically repaint when the game's state
 * changes. Therefore, another object must call repaint() whenever
 * necessary.
 */
public class GameView extends Component {
	private RTflowCode rtflowCode;
	private Image starField;
	private int x1, y1, rot1, x2, y2, rot2;
	private int bulletx1, bullety1, bulletx2, bullety2;
	private int bullet1, bullet2;
	private int thrust1, thrust2;
	private int hits1on2, hits2on1;
	private boolean[] boolArray;
	private int[] intArray;
	private float[] realArray;
	
	/**
	 * Constructor.
	 * @param rtflowCode The object from which the game's state should be read.
	 */
	public GameView(RTflowCode rtflowCode) {
		this.rtflowCode = rtflowCode;
		this.starField = null;
		boolArray = rtflowCode.getBoolArray();
		intArray = rtflowCode.getIntArray();
		realArray = rtflowCode.getRealArray();
		
		// Find indexes of output variables in generated code
		x1 = RTflowCode.findName(rtflowCode.realNames, "x1");
		y1 = RTflowCode.findName(rtflowCode.realNames, "y1");
		rot1 = RTflowCode.findName(rtflowCode.realNames, "rot1");
		x2 = RTflowCode.findName(rtflowCode.realNames, "x2");
		y2 = RTflowCode.findName(rtflowCode.realNames, "y2");
		rot2 = RTflowCode.findName(rtflowCode.realNames, "rot2");
		bulletx1 = RTflowCode.findName(rtflowCode.realNames, "bulletx1");
		bullety1 = RTflowCode.findName(rtflowCode.realNames, "bullety1");
		bulletx2 = RTflowCode.findName(rtflowCode.realNames, "bulletx2");
		bullety2 = RTflowCode.findName(rtflowCode.realNames, "bullety2");
		bullet1 = RTflowCode.findName(rtflowCode.boolNames, "bullet1");
		bullet2 = RTflowCode.findName(rtflowCode.boolNames, "bullet2");
		thrust1 = RTflowCode.findName(rtflowCode.boolNames, "thrust1");
		thrust2 = RTflowCode.findName(rtflowCode.boolNames, "thrust2");
		hits1on2 = RTflowCode.findName(rtflowCode.intNames, "hits1on2");
		hits2on1 = RTflowCode.findName(rtflowCode.intNames, "hits2on1");
	}
	
	/**
	 * Repaints the component.
	 * @param g Graphic context.
	 */
	public void paint(Graphics g) {
		// The first time, a star field is painted to buffered image starField.
		// This can not be done in the constructor, because there is no
		// Graphics object associated to the component until it becomes
		// visible. That is, in the constructor, createImage() would return
		// null.
		if (starField == null) {
			starField = createImage(640, 480);
			Graphics graphics = starField.getGraphics();
			graphics.setColor(Color.BLACK);
			graphics.fillRect(0, 0, 640, 480);
			graphics.setColor(Color.WHITE);
			((Graphics2D)graphics).setPaint(Color.WHITE);
			for (int i = 0; i < 1000; i++) {
				int x = (int)(640 * Math.random());
				int y = (int)(480 * Math.random());
				graphics.drawOval(x, y, 1, 1);
				if (Math.random() < 0.1) {
					graphics.drawOval(x, y, 2, 2);
				}
			}
		}
		
		// Copy buffered image to component surface
		g.drawImage(starField, 0, 0, null);
		
		// Paint game elements
		g.setColor(Color.WHITE);
		paintShip(g, realArray[x1], realArray[y1], realArray[rot1], boolArray[thrust1]);
		paintShip(g, realArray[x2], realArray[y2], realArray[rot2], boolArray[thrust2]);
		if (boolArray[bullet1]) {
			paintBullet(g, realArray[bulletx1], realArray[bullety1]);
		}
		if (boolArray[bullet2]) {
			paintBullet(g, realArray[bulletx2], realArray[bullety2]);
		}
		
		// Paint score
		g.setFont(g.getFont().deriveFont(40f));
		g.drawString(String.valueOf(intArray[hits1on2]), 10, 440);
		g.drawString(String.valueOf(intArray[hits2on1]), 600, 440);
	}
	
	/**
	 * Paints a simple spaceship.
	 * @param g Graphics context.
	 * @param x X coordinate of center of spaceship.
	 * @param y Y coordinate of center of spaceship.
	 * @param r Direction of spaceship in radians counterclokwise from x axis.
	 * @param thrust True to show flame in the back of spaceship, indicating thrust.
	 */
	private void paintShip(Graphics g, double x, double y, double r, boolean thrust) {
		// The points of the polygon that constitutes the spaceship at a 0 radians
		// angle, that is, pointing to the right. Normally, only the first four
		// points are used, but when thrust is true, all points are used, which
		// extends the polygon with a flame in the back.
		double[] xo = { -20, 20, -20, -10, -20, -17, -22, -17, -20, -10 }; 
		double[] yo = {  15,  0, -15,   0,   5,   2,   0,  -2,  -5,   0 };
		int n = thrust ? Math.min(xo.length, yo.length) : 4;
		
		// Rotate points
		int[] xt = new int[n];
		int[] yt = new int[n];
		double cosr = Math.cos(r);
		double sinr = Math.sin(r);
		for (int i = 0; i < n; i++) {
			xt[i] = (int)(cosr * xo[i] - sinr * yo[i] + x);
			yt[i] = (int)(-sinr * xo[i] - cosr * yo[i] + y);
		}
		
		// Paint polygon
		g.fillPolygon(xt, yt, n);
	}

	/**
	 * Paints a bullet.
	 * @param g Graphics context.
	 * @param x X coordinate of center of bullet.
	 * @param y Y coordinate of center of bullet.
	 */
	private void paintBullet(Graphics g, double x, double y) {
		g.fillOval((int)x - 2, (int)y, 5, 5);
	}
}
